##############################################################################
#
# Enfold Enterprise Deployment - Remote Deployment of Content
# Copyright (C) 2005 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
$Id: __init__.py 732 2005-01-21 19:43:40Z sidnei $
"""
import types
import os
import os.path
import sys

def copy_file(infile, outfile):
    """ Read binary data from infile and write it to outfile. infile
    and outfile may be strings, in which case a file with that name is
    opened, or filehandles, in which case they are accessed directly.
    """

    if type(infile) is types.StringType:
        instream = open(infile, 'rb')
        read_in = instream.read
        close_in = instream.close
        seek_in = instream.seek
    elif type(infile) is types.IntType:
        # Maybe a file descriptor?
        def read(bytes):
            return os.read(infile, bytes)
        read_in = read
        def close():
            return os.close(infile)
        close_in = close
        def seek(pos, how):
            return os.lseek(infile, pos, how)
        seek_in = seek
    else:
        close_in = 0
        instream = infile
        read_in = instream.read
        seek_in = instream.seek

    if type(outfile) is types.StringType:
        outstream = open(outfile, 'wb')
        write_out = outstream.write
        close_out = outstream.close
    elif type(outfile) is types.IntType:
        # Maybe a file descriptor?
        def write(bytes):
            return os.write(outfile, bytes)
        write_out = write
        def close():
            return os.close(outfile)
        close_out = close
    else:
        close_out = 0
        outstream = outfile
        write_out = outstream.write

    try:
        blocksize = 2 << 16
        block = read_in(blocksize)
        write_out(block)
        while len(block) == blocksize:
            block = read_in(blocksize)
            write_out(block)
        seek_in(0)
    finally:
        if close_in: close_in()
        if close_out: close_out()

def make_target_path(originals, target):
    path = os.path.splitdrive(target)[-1]
    path = path.strip(os.path.sep)
    parts = os.path.split(path)
    destination = os.path.join(originals, *parts)
    return destination

def path2url(path):
    return path.replace(os.path.sep, '/')

def caller(start=1, end=1):
    """Return the name of the calling function n levels up in the frame stack.
    Ex: caller(0) ==> 'caller'; def f(): return caller(); f() ==> 'f'
    """
    retval = []
    import inspect
    for f in inspect.getouterframes(inspect.currentframe())[start:end]:
        retval.append(f[3])
        del f
    retval.reverse()
    return ' . '.join(retval)

def package_home(globals_dict):
    __name__ = globals_dict['__name__']
    m = sys.modules[__name__]
    if hasattr(m, '__path__'):
        r = m.__path__[0]
    elif "." in __name__:
        r = sys.modules[__name__[:__name__.rfind('.')]].__path__[0]
    else:
        r = __name__
    return os.path.abspath(r)

class Utility:

    path = os.path

    def __init__(self, data_prefix, entransit_prefix,
                 package_prefix=None, globals_dict=None):
        if package_prefix is None:
            assert globals_dict is not None
            package_prefix = package_home(globals_dict)
        self.package_prefix = package_prefix
        self.data_prefix = data_prefix
        self.entransit_prefix = entransit_prefix
        self.data_path = self.package_path(*self.data_prefix)

    def join(self, *parts):
        return self.path.sep.join(parts)

    def content_path(self, path):
        if isinstance(path, (tuple, list)):
            path = self.join(*path)
        path = self.translate_path(path)
        if path.startswith(self.data_path):
            return path
        parts = self.data_prefix + tuple(path.split('/'))
        path = self.package_path(*parts)
        return path

    def translate_path(self, p):
        p = p.replace(self.entransit_prefix, '').strip(self.path.sep)
        return '/' + '/'.join(filter(None, p.split(self.path.sep)))

    def package_path(self, *args):
        return self.join(self.package_prefix, *args)

    def metadata_path(self, path):
        path = self.content_path(path)
        path.strip(self.path.sep)
        if not path.endswith('.data'):
            path += '.data'
        return path

    def exists(self, path):
        if isinstance(path, unicode):
            path = path.encode('utf-8')
        return (self.path.isfile(path) and
                self.path.exists(path)) and path or False

    def isdir(self, path):
        if isinstance(path, unicode):
            path = path.encode('utf-8')
        return (self.path.isdir(path) and
                self.path.exists(path)) and path or False

    def get_metadata(self, path):
        from xmlrpclib import loads
        path = self.exists(self.metadata_path(path))
        if not path:
            return {}
        return loads(open(path, 'rb').read())[0][0]
